"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const tmp = __importStar(require("tmp"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const flamegrill_1 = require("../flamegrill");
const process_1 = require("../process/process");
const regression_1 = require("../analyze/regression");
// TODO: these are black box tests for now but should be refactored to be unit tests
// TODO: modules that output files should be modified not to and wrapped by a centralized file output helper
// TODO: consider also making file output / github / CI integration another package within this repo
describe('flamegrill', () => {
    describe('cook', () => {
        const profiles = require('../fixtures/profiles.json');
        const util = require('../util');
        const fixturesDir = path_1.default.join(__dirname, '../fixtures');
        let scenarios = {};
        let scenarioNames;
        let outdir;
        let scenarioConfig;
        let scenarioIndex = 0;
        const testPage = {
            close: jest.fn(() => Promise.resolve()),
            goto: jest.fn(() => {
                return Promise.resolve(null);
            }),
            metrics: jest.fn(() => Promise.resolve({})),
            setDefaultTimeout: jest.fn(() => { })
        };
        const testBrowser = {
            newPage: jest.fn(() => {
                return Promise.resolve(testPage);
            }),
            close: jest.fn(() => { })
        };
        beforeAll(() => {
            // Processing all profiles takes more than the default 5 seconds.
            jest.setTimeout(60000);
            jest.spyOn(puppeteer_1.default, 'launch').mockImplementation(() => {
                return Promise.resolve(testBrowser);
            });
            scenarioNames = Object.keys(profiles);
            scenarioNames.forEach(scenarioName => {
                scenarios[scenarioName] = {
                    scenario: scenarioName + ".url",
                    // TODO: changing this value doesn't matter. is this setup needed?
                    baseline: scenarioName + "_base.url"
                };
            });
        });
        beforeEach(() => {
            jest.clearAllMocks();
            outdir = tmp.dirSync({ unsafeCleanup: true });
            scenarioConfig = {
                outDir: outdir.name,
                tempDir: fixturesDir
            };
        });
        afterAll(() => {
            jest.resetAllMocks();
            outdir.removeCallback();
        });
        it('generates expected output', () => __awaiter(this, void 0, void 0, function* () {
            const snapshotsDir = path_1.default.join(__dirname, '../fixtures/snapshots');
            const expectedResults = require('../fixtures/results.json');
            // Set up arr_diff to feed in predefined profiler log fixtures.
            jest.spyOn(util, 'arr_diff').mockImplementation(() => {
                let logfile;
                // This code assumes scenarios are processed in the same order passed into cook, which
                // is true as of writing.
                const scenarioName = scenarioNames[Math.floor(scenarioIndex / 2)];
                if (scenarioIndex % 2) {
                    logfile = profiles[scenarioName].baseline.logFile;
                }
                else {
                    logfile = profiles[scenarioName].logFile;
                }
                scenarioIndex += 1;
                return [logfile];
            });
            const testResults = yield flamegrill_1.cook(scenarios, scenarioConfig);
            // The path will differ for every test run, so remove it before comparing results.
            removePaths(testResults);
            // Convenience line left commented out for updating expected output.
            // fs.writeFileSync(path.join(outdir.name, "results.json"), JSON.stringify(testResults));
            expect(testResults).toEqual(expectedResults);
            const snapshotFiles = fs_1.default.readdirSync(snapshotsDir);
            const testFiles = fs_1.default.readdirSync(outdir.name);
            expect(testFiles).toEqual(snapshotFiles);
            testFiles.forEach(file => {
                // Some generated output creates files with \r\n. Some environments spit out \n.
                // Ignore line break types when comparing results.
                const analysis = fs_1.default.readFileSync(path_1.default.join(snapshotsDir, file), 'utf8').split(/\r?\n/g);
                const output = fs_1.default.readFileSync(path_1.default.join(outdir.name, file), 'utf8').split(/\r?\n/g);
                expect(output).toEqual(analysis);
            });
            expect(testBrowser.close.mock.calls.length).toEqual(1);
        }));
        it('errors on invalid profile logs', () => __awaiter(this, void 0, void 0, function* () {
            const expectedResults = require('../fixtures/errors/errors.json');
            // Set up arr_diff to feed in predefined profiler log fixtures.
            jest.spyOn(util, 'arr_diff').mockImplementation(() => {
                let logfile = "errors/error.prof";
                return [logfile];
            });
            const testResults = yield flamegrill_1.cook({ 'test': { scenario: 'testUrl' } }, scenarioConfig);
            // The path will differ for every test run, so remove it before comparing results.
            removePaths(testResults);
            // Convenience line left commented out for updating expected output.
            // fs.writeFileSync(path.join(outdir.name, "errors.json"), JSON.stringify(testResults));
            expect(testResults).toEqual(expectedResults);
            const testFiles = fs_1.default.readdirSync(outdir.name);
            expect(testFiles).toEqual(['test.err.txt']);
            const errorFile = fs_1.default.readFileSync(path_1.default.join(outdir.name, 'test.err.txt'));
            expect(errorFile.includes('Error: dispatchLogRow_: Can\'t parse tick,0xfffffffdeb11c0e4,55914,0,0x0,6, integer too large.')).toBeTruthy();
            expect(testBrowser.close.mock.calls.length).toEqual(1);
        }));
    });
    // These tests are technically redundant with cook tests but are left for now as they may be useful 
    // as unit tests are added.
    describe('generateFlamegraph, findRegressions', () => {
        const { processProfile } = process_1.__unitTestHooks;
        const profiles = require('../fixtures/profiles.json');
        const snapshotsDir = path_1.default.join(__dirname, '../fixtures/snapshots');
        it('generates expected output', () => __awaiter(this, void 0, void 0, function* () {
            // Processing all profiles takes more than the default 5 seconds.
            jest.setTimeout(30000);
            expect.assertions(89);
            const outdir = tmp.dirSync({ unsafeCleanup: true });
            // TODO: replace with processProfiles?
            yield Promise.all(Object.keys(profiles).map(key => {
                let logfile = require.resolve(path_1.default.join('../fixtures', profiles[key].logFile));
                let outfile = path_1.default.join(outdir.name, key);
                return processProfile(logfile, outfile);
            }));
            yield Promise.all(Object.keys(profiles).map(key => {
                let logfile = require.resolve(path_1.default.join('../fixtures', profiles[key].baseline.logFile));
                let outfile = path_1.default.join(outdir.name, key + '_base');
                return processProfile(logfile, outfile);
            }));
            Object.keys(profiles).forEach(key => {
                // TODO: this code block is duplicating code in flamegrill.ts and should be removed as code is refactored.
                let datafileBefore = path_1.default.join(outdir.name, key + '_base.data.js');
                let datafileAfter = path_1.default.join(outdir.name, key + '.data.js');
                let regressionfile = path_1.default.join(outdir.name, key + '.regression.txt');
                const analysis = regression_1.findRegressions(datafileBefore, datafileAfter);
                if (analysis.isRegression) {
                    fs_1.default.writeFileSync(regressionfile, analysis.summary);
                }
            });
            const snapshotFiles = fs_1.default.readdirSync(snapshotsDir);
            const testFiles = fs_1.default.readdirSync(outdir.name);
            expect(testFiles).toEqual(snapshotFiles);
            testFiles.forEach(file => {
                // Some generated output creates files with \r\n. Some environments spit out \n.
                // Ignore line break types when comparing results.
                const analysis = fs_1.default.readFileSync(path_1.default.join(snapshotsDir, file), 'utf8').split(/\r?\n/g);
                const output = fs_1.default.readFileSync(path_1.default.join(outdir.name, file), 'utf8').split(/\r?\n/g);
                expect(output).toEqual(analysis);
            });
            outdir.removeCallback();
        }));
    });
});
/**
 * Helper to remove paths from filenames, leaving just base filename.
 */
function removePaths(obj) {
    Object.keys(obj).forEach(key => {
        if (key.includes('File')) {
            console.log(`key = ${key}`);
            obj[key] = path_1.default.basename(obj[key]);
        }
        else if (obj[key] instanceof Object) {
            removePaths(obj[key]);
        }
    });
}
